# Generated by Django 5.1.4 on 2025-04-20 23:04

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0011_alter_object_name_alter_object_unique_name_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='Preposition',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='ArgumentSpecifier',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('prepositionSpecifier', models.CharField(blank=True, choices=[('any', 'any'), ('none', 'none')], db_index=True, max_length=255, null=True)),
                ('objectSpecifier', models.CharField(choices=[('this', 'this'), ('any', 'any'), ('none', 'none')], db_index=True, max_length=255)),
                ('preposition', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='core.preposition')),
            ],
        ),
        migrations.CreateModel(
            name='PrepositionName',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('preposition', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='names', to='core.preposition')),
            ],
        ),
    ]
