#!moo verb disfunc --on $player

# pylint: disable=return-outside-function,undefined-variable

"""
This verb is invoked when the player disconnects from the LambdaMOO server. The verb defined for the `$player` class
does nothing. You might choose to override this verb to print a special message when you log out, or perhaps to
retrieve some of your possessions.
"""

pass
