# Generated by Django 4.2.5 on 2023-11-27 02:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0002_alter_object_observers_alter_object_unique_name"),
    ]

    operations = [
        migrations.AlterField(
            model_name="verb",
            name="ability",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="verb",
            name="method",
            field=models.BooleanField(default=False),
        ),
    ]
